package com.yizhi.sign.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.sign.application.vo.domain.StatisticsSign;
import com.yizhi.sign.application.vo.domain.StatisticsSignLearn;
import com.yizhi.sign.application.vo.domain.TrSign;
import com.yizhi.sign.application.vo.domain.TrSignTime;
import com.yizhi.util.application.domain.Response;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Date;
import java.util.List;

@FeignClient(name = "sign", contextId = "ReportSignClient")
public interface ReportSignClient {

    /**
     * 异步加载统计
     *
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/remote/statisticsSign/asynchronous/load/data")
    public Response<String> asynchronousSign(@RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate);

    @GetMapping("/remote/statisticsSign/group")
    Page<StatisticsSign> signPage(@RequestParam(name = "projectId", required = false) Long projectId,
                                  @RequestParam(name = "kwd", required = false) String kwd,
                                  @RequestParam(name = "startDate", required = false) String startDate,
                                  @RequestParam(name = "endDate", required = false) String endDate,
                                  @RequestParam(name = "signCompanyId", required = false) Long signCompanyId,
                                  @RequestParam(name = "signOrgIds", required = false) List<Long> signOrgIds,
                                  @RequestParam(name = "signSiteId", required = false) Long signSiteId,
                                  @RequestParam(name = "pageNo", required = false) Integer pageNo,
                                  @RequestParam(name = "pageSize", required = false) Integer pageSize);

    @GetMapping("/remote/statisticsSign/group/view")
    Page<StatisticsSign> signView(
            @RequestParam(name = "id", required = true) Long id,
            @RequestParam(name = "projectId", required = true) Long projectId,
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "orgNameOrCode", required = false) String orgNameOrCode,
            @RequestParam(name = "accountName", required = false) String accountName,
            @RequestParam(name = "pageNo", required = false) Integer pageNo,
            @RequestParam(name = "pageSize", required = false) Integer pageSize);

    @GetMapping("/remote/statisticsSign/chart/group")
    List<StatisticsSign> chartGroup(@RequestParam(name = "trainningProjectId", required = false) Long trainningProjectId,
                                    @RequestParam(name = "startDate", required = false) String startDate,
                                    @RequestParam(name = "endDate", required = false) String endDate,
                                    @RequestParam(name = "signCompanyId", required = false) Long signCompanyId,
                                    @RequestParam(name = "signOrgIds", required = false) List<Long> signOrgIds,
                                    @RequestParam(name = "signSiteId", required = false) Long signSiteId);

    @ApiOperation(value = "各项目的签到统计", notes = "各项目的签到统计")
    @GetMapping("/remote/statisticsSign/group/count")
    Integer signPageCount(@RequestParam(name = "projectId", required = false) Long projectId,
                          @RequestParam(name = "startDate", required = false) String startDate,
                          @RequestParam(name = "endDate", required = false) String endDate,
                          @RequestParam(name = "signCompanyId", required = false) Long signCompanyId,
                          @RequestParam(name = "signOrgIds", required = false) List<Long> signOrgIds,
                          @RequestParam(name = "signSiteId", required = false) Long signSiteId);

    @ApiOperation(value = "各项目的签到统计列表", notes = "各项目的签到统计")
    @GetMapping("/remote/statisticsSign/group/list")
    List<StatisticsSign> signPageList(@RequestParam(name = "projectId", required = false) Long projectId,
                                      @RequestParam(name = "startDate", required = false) String startDate,
                                      @RequestParam(name = "endDate", required = false) String endDate,
                                      @RequestParam(name = "signCompanyId", required = false) Long signCompanyId,
                                      @RequestParam(name = "signOrgIds", required = false) List<Long> signOrgIds,
                                      @RequestParam(name = "signSiteId", required = false) Long signSiteId,
                                      @RequestParam(name = "pageNo", required = false) Integer pageNo,
                                      @RequestParam(name = "pageSize", required = false) Integer pageSize);

    @ApiOperation(value = "某个项目的签到统计个数", notes = "某个项目的签到统计个数")
    @GetMapping("/remote/statisticsSign/group/view/count")
    Integer signViewCount(
            @RequestParam(name = "id", required = true) Long id,
            @RequestParam(name = "projectId", required = true) Long projectId,
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "orgNameOrCode", required = false) String orgNameOrCode,
            @RequestParam(name = "accountName", required = false) String accountName);

    @ApiOperation(value = "某个项目的签到统计个数列表", notes = "某个项目的签到统计个数列表")
    @GetMapping("/remote/statisticsSign/group/view/list")
    List<StatisticsSign> signViewList(
            @RequestParam(name = "id", required = true) Long id,
            @RequestParam(name = "projectId", required = true) Long projectId,
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "orgNameOrCode", required = false) String orgNameOrCode,
            @RequestParam(name = "accountName", required = false) String accountName,
            @RequestParam(name = "pageNo", required = false) Integer pageNo,
            @RequestParam(name = "pageSize", required = false) Integer pageSize);

    @ApiOperation(value = "获取签到时间段", notes = "获取签到时间段")
    @GetMapping("/remote/statisticsSign/group/getSignTime")
    TrSignTime getSignTime(@RequestParam("getSignTime") Long signTimeId);

    @GetMapping("/remote/statisticsSign/group/selectRecordMinTime")
    Date selectRecordMinTime();

    @GetMapping("/remote/statisticsSign/group/getAllResearchs")
    List<TrSign> getAllResearchs(@RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate);

    /**
     * 报表服务获取签到记录
     * @param signTimeId
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/remote/statisticsSign/group/getRecords")
    List<StatisticsSignLearn> getRecords(@RequestParam("signTimeId") Long signTimeId, @RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate);

}
