package com.yizhi.sign.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.sign.application.vo.domain.TrSignAccount;
import com.yizhi.sign.application.vo.*;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Map;

/**
 * 签到-学员端 <br>
 * 〈〉
 *
 * @author phone
 * @create 2018/4/12
 * @since 1.0.0
 */
@FeignClient(name = "sign", contextId = "SignApiClient")
public interface SignApiClient {

    /**
     * 学员端二维码签到
     * @param signIdVO 
     * @return
     */
    @PostMapping(value = "/remote/api/sign/", produces="text/html;charset=UTF-8")
    String sign(@RequestBody SignVO signVO);

    /**
     * 我的签到记录
     * @param pageVO
     * @return
     */
    @PostMapping("/remote/api/sign/my/record/list")
    Page<TrSignRecordVo> recordList(@RequestBody PageVO pageVO);


    /**
     *  获取签到总数量( PC 端   培训项目中培训统计的签到总数)
     * @param trainingProjectId
     * @return
     */
    @PostMapping("/remote/api/sign/pc/count")
    public Integer getPcSignTimeCount(@RequestParam("trainingProjectId") Long trainingProjectId);

	@ApiOperation(value = "验证签到邀请码", notes = "验证签到邀请码", response = TrSignAccount.class)
	@GetMapping("/remote/api/sign/checkCode")
	public Map<String, Object> checkCode(
			@ApiParam(name = "code", value = "项目签到邀请码", required = false) @RequestParam(name = "code", required = true) String code,
			@ApiParam(name = "openId", value = "微信小程序openId 默认不传", required = false) @RequestParam(name = "openId", required = false) String openId,
			@ApiParam(name = "accountId", value = "用户id 默认不传", required = false) @RequestParam(name = "accountId", required = false) Long accountId);
	
	@ApiOperation(value = "验证用户身份", notes = "验证用户身份")
	@PostMapping("/remote/api/sign/checkAccount")
	public Map<String, Object> checkAccount(@RequestBody SignAccountParam param);
	
	@ApiOperation(value = "小程序签到详情", notes = "小程序签到详情", response = ApiSignTimeVO.class)
	@GetMapping("/remote/api/sign/signDetail")
	public ApiSignTimeVO signDetail(
			@ApiParam(name = "accountId", value = "用户id ", required = true) @RequestParam(name = "accountId", required = true) Long accountId,
			@ApiParam(name = "code", value = "项目签到邀请码", required = true) @RequestParam(name = "code", required = true) String code,
			@ApiParam(name = "trainingProjectId", value = "培训项目id ", required = false) @RequestParam(name = "trainingProjectId", required = false) Long trainingProjectId);
	
	@ApiOperation(value = "小程序签到切换用户", notes = "小程序签到切换用户")
	@GetMapping("/remote/api/sign/delRelation")
	public String delRelation(
			@ApiParam(name = "accountId", value = "用户id ", required = true) @RequestParam(name = "accountId", required = true) Long accountId,
			@ApiParam(name = "code", value = "项目签到邀请码", required = true) @RequestParam(name = "code", required = true) String code,
			@ApiParam(name = "openId", value = "微信小程序openId", required = true) @RequestParam(name = "openId", required = true) String openId);
	
	@GetMapping("/remote/api/sign/insert")
	public String insert(
			@ApiParam(name = "accountId", value = "用户id ", required = true) @RequestParam(name = "accountId", required = true) Long accountId,
			@ApiParam(name = "trainingProjectId", value = "项目id ", required = true) @RequestParam(name = "trainingProjectId", required = true) Long trainingProjectId,
			@ApiParam(name = "signTimeId", value = "签到id ", required = true) @RequestParam(name = "signTimeId", required = true) Long signTimeId);
}