package com.yizhi.application.controller.api;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.training.application.vo.domain.TpAuthorizationRangeVo;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import com.yizhi.point.application.feign.PointRedisFeignClients;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.service.SignAccountService;
import com.yizhi.application.service.SignRecordService;
import com.yizhi.application.service.SignService;
import com.yizhi.application.service.SignTimeService;
import com.yizhi.system.application.vo.domain.Account;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.util.application.encrypt.ShaEncrypt;
import com.yizhi.util.application.json.JsonUtil;
import com.yizhi.application.domain.TrSignAccount;
import com.yizhi.application.domain.TrSignTime;
import com.yizhi.training.application.feign.TrainingProjectClient;
import com.yizhi.sign.application.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * <p>
 * 签到主体表，入口在培训项目 前端控制器 学员端
 * </p>
 *
 * @author fanchunhui123
 * @since 2018-03-07
 */
@Api(tags = "签到-学员端", description = "签到-学员端")
@RestController
@RequestMapping("/api/sign")
public class SignController {

	private static final Logger LOG = LoggerFactory.getLogger(SignController.class);

	@Autowired
	private SignService signService;

	@Autowired
	private TaskExecutor taskExecutor;

	@Autowired
	private SignTimeService signTimeService;

	@Autowired
	private SignAccountService signAccountService;

	@Autowired
	private TrainingProjectClient trainingProjectClient;

	@Autowired
	private AccountClient accountClient;

	@Autowired
	IdGenerator idGenerator;

	@Autowired
	private SignRecordService signRecordService;

	@Autowired
	private PointRedisFeignClients pointRedisFeignClients;
/*
	@Autowired
	private AmqpTemplate amqpTemplate;*/

	/**
	 * 学员端二维码签到
	 *
	 * @param signVO
	 * @return
	 */
	@PostMapping(value = "/", produces = "text/html;charset=UTF-8")
	public String sign(@RequestBody SignVO signVO) {
		try {
			if (null == signVO) {
				return "不存在此签到";
			}
			if (null == signVO.getTrainingProjectId() || null == signVO.getSignTimeId()
					|| null == signVO.getAccountId()) {
				return "不存在此签到";
			}

			// 查询是否签到
			String result = signService.sign(signVO.getAccountId(), String.valueOf(signVO.getTrainingProjectId()),
					String.valueOf(signVO.getSignTimeId()), signVO.getSignType());
			LOG.info(result);

			return result;

		} catch (Exception e) {
			LOG.error("", e);
			return "不存在此签到";
		}
	}

	/**
	 * 我的签到记录
	 *
	 * @param pageVO
	 * @return
	 */
	@PostMapping("/my/record/list")
	public Page<TrSignRecordVo> recordList(@RequestBody PageVO pageVO) {
		try {
			// TODO 查询签到端
			// accountId 学员账号id
			Long accountId = pageVO.getAccountId();
			String year = pageVO.getYear();
			Calendar calendar = null;
//			if (StringUtils.isBlank(year)) {
//				calendar = Calendar.getInstance();
//				calendar.setTime(new Date());
//				year = String.valueOf(calendar.get(Calendar.YEAR));
//			}
//			else {
//				year = year.trim();
//			}
			if (StringUtils.isNotBlank(year)) {
				year = year.trim();
			}
			String month = pageVO.getMonth();
			if (!StringUtils.isBlank(month)) {
				month = month.trim();
			}
			Integer pageNo = pageVO.getPageNo();
			Integer pageSize = pageVO.getPageSize();
			if (null == pageNo || pageNo == 0) {
				pageNo = 1;
			}
			if (null == pageSize) {
				pageSize = 10;
			}
			Page<TrSignRecordVo> page = signService.selectSignRecordListByAccountIdAndMonth(accountId, year, month,
					pageNo, pageSize);
			/*
			 * Map<String, Integer> pageMap = new HashMap<String, Integer>();
			 * pageMap.put("pageNo", pageVO.getPageNo()); pageMap.put("pageSize",
			 * pageVO.getPageSize()); pageMap.put("pageTotal", page.getTotal());
			 *
			 * Map<String, Object> resap = new HashMap<String, Object>(); resap.put("list",
			 * page.getRecords()); resap.put("page", pageMap);
			 */

			return page;
		} catch (Exception e) {
			LOG.error("", e);
			return null;
		}
	}

	/********************************************************************
	 * PC 端
	 ************************************************************************/

	/**
	 * 获取签到总数量( PC 端 培训项目中培训统计的签到总数)
	 *
	 * @param trainingProjectId
	 * @return
	 */
	@PostMapping("/pc/count")
	public Integer getPcSignTimeCount(@RequestParam("trainingProjectId") Long trainingProjectId) {
		Integer count = signService.getPcSignTimeCount(trainingProjectId);
		return count;

	}

	@ApiOperation(value = "验证签到邀请码", notes = "验证签到邀请码", response = TrSignAccount.class)
	@GetMapping("/checkCode")
	public Map<String, Object> checkCode(
			@ApiParam(name = "code", value = "项目签到邀请码", required = false) @RequestParam(name = "code", required = true) String code,
			@ApiParam(name = "openId", value = "微信小程序openId 默认不传", required = false) @RequestParam(name = "openId", required = false) String openId,
			@ApiParam(name = "accountId", value = "用户id 默认不传", required = false) @RequestParam(name = "accountId", required = false) Long accountId) {
		Map<String, Object> map = new HashMap<String, Object>();
		String result = "";
		TrSignTime signTime = new TrSignTime();
		signTime.setCode(code);
		signTime.setDeleted(0);
		TrSignTime one = signTimeService.selectOne(new EntityWrapper<TrSignTime>(signTime));
		if (one != null) {
			Date endTime = one.getEndTime();
			long day = (System.currentTimeMillis() - endTime.getTime()) / (60 * 60 * 24 * 1000);
			if ((System.currentTimeMillis() - endTime.getTime()) > 0 && day >= 0 && day <= 7) {
				result = "签到已过期";
				map.put("error", result);
				return map;
			}
			if (day > 7) {
				result = "无效邀请码,请重新输入";
				map.put("error", result);
				return map;
			}
			TrSignAccount account = new TrSignAccount();
			account.setCode(code);
			if (openId != null) {
				account.setOpenId(openId);
			}
			if (accountId != null) {
				account.setAccountId(accountId);
			}
			account.setDeleted(0);
			TrSignAccount signAccount = signAccountService.selectOne(new EntityWrapper<TrSignAccount>(account));
			if (signAccount != null) {
				map.put("success", JsonUtil.object2Json(signAccount));
				return map;
			} else {
				return map;
			}
		}
		result = "无效邀请码，请重新输入";
		map.put("error", result);
		return map;
	}

	@ApiOperation(value = "验证用户身份", notes = "验证用户身份", response = Account.class)
	@PostMapping("/checkAccount")
	public Map<String, Object> checkAccount(@RequestBody SignAccountParam param) {
		Integer result = 0;
		String error = "";
		Map<String, Object> map = new HashMap<String, Object>();
		Date date = new Date();
		TrSignTime signTime = new TrSignTime();
		signTime.setCode(param.getCode());
		signTime.setDeleted(0);
		TrSignTime one = signTimeService.selectOne(new EntityWrapper<TrSignTime>(signTime));
		if (one != null) {
			Date endTime = one.getEndTime();
			if (date.after(endTime)) {
				error = "签到已过期";
				map.put("error", error);
				return map;
			}
			Long tpId = one.getTrainingProjectId();
			TrainingProjectVo tProject = trainingProjectClient.getOne(tpId);
			if (tProject == null) {
				LOG.error(tpId + "培训项目不存在！！！");
				error = "培训项目不存在";
				map.put("error", error);
				return map;
			}
			LOG.info("返回培训项目对象实体" + tProject.toString());
			Long companyId = tProject.getCompanyId();
			Account user = accountClient.findAccountByName(param.getName(), companyId);
			if (user == null) {
				LOG.error(param.getName() + "用户不存在");
				map.put("error", "账号或密码输入错误！");
				return map;
			}
			if (!ShaEncrypt.comparePassword(param.getPwd(), user.getPassword())) {
				LOG.info("用户密码错误，加密密码=" + user.getPassword() + "输入密码=" + param.getPwd());
				map.put("error", "账号或密码输入错误！");
				return map;
			}
			if (!user.getEnabled() || user.getLocked()) {
				// 账号未启用或者锁定
				map.put("error", "账号或密码输入错误！");
				return map;
				// 前面时间大于后面时间==1
			} else if (user.getExpiredType() == 2 && user.getEndTime().compareTo(new Date()) <= 0) {
				// 账号已过期
				map.put("error", "账号或密码输入错误！");
				return map;
			} else if (user.getExpiredType() == 3 && user.getExpiredTime().compareTo(new Date()) <= 0) {
				map.put("error", "账号或密码输入错误！");
				return map;
			}
			Integer range = tProject.getVisibleRange();
			if (range == 1) {
				result = 1;
			} else if (range == 0) {
				List<Long> accountIds = new ArrayList<>();
				List<TpAuthorizationRangeVo> visibleRanges = trainingProjectClient.VisibleRange(tpId);
				for (TpAuthorizationRangeVo visibleRange : visibleRanges) {
					if (2 == visibleRange.getType()) {
						accountIds.add(visibleRange.getRelationId());
					} else {
						List<AccountVO> list = accountClient.findByOrgId(visibleRange.getRelationId(), companyId);
						List<Long> ids = new ArrayList<Long>();
						if (CollectionUtils.isNotEmpty(list)) {
							for (AccountVO vo : list) {
								ids.add(vo.getId());
							}
						}
						if (CollectionUtils.isNotEmpty(ids)) {
							accountIds.addAll(ids);
						}
					}
				}
				if (CollectionUtils.isEmpty(accountIds)) {
					error = "您没有权限";
					map.put("error", error);
					return map;
				}
				result = accountIds.contains(user.getId()) ? 1 : 0;
			}
			if (result.equals(0)) {
				error = "您没有权限";
				map.put("error", error);
				return map;
			}
			TrSignAccount signAccount = new TrSignAccount();
			signAccount.setId(idGenerator.generate());
			signAccount.setAccountId(user.getId());
			signAccount.setCode(param.getCode());
			signAccount.setOpenId(param.getOpenId());
			signAccount.setDeleted(0);
			signAccount.setFullName(user.getFullName());
			signAccount.setCreateById(user.getId());
			signAccount.setCreateByName(user.getName());
			signAccount.setCreateTime(date);
			signAccountService.insert(signAccount);
			map.put("success", JsonUtil.object2Json(user));
			return map;
		}
		LOG.error(param.getCode() + "无效邀请码");
		error = "无效邀请码,请重新输入";
		map.put("error", error);
		return map;
	}

	@ApiOperation(value = "小程序签到详情", notes = "小程序签到详情", response = ApiSignTimeVO.class)
	@GetMapping("/signDetail")
	public ApiSignTimeVO signDetail(
			@ApiParam(name = "accountId", value = "用户id ", required = true) @RequestParam(name = "accountId", required = true) Long accountId,
			@ApiParam(name = "code", value = "项目签到邀请码", required = true) @RequestParam(name = "code", required = true) String code,
			@ApiParam(name = "trainingProjectId", value = "培训项目id ", required = false) @RequestParam(name = "trainingProjectId", required = false) Long trainingProjectId) {
		ApiSignTimeVO vo = signService.getSignTimeDeatil(accountId, code, trainingProjectId);
		return vo;
	}

	@ApiOperation(value = "小程序签到切换用户", notes = "小程序签到切换用户")
	@GetMapping("/delRelation")
	public String delRelation(
			@ApiParam(name = "accountId", value = "用户id ", required = true) @RequestParam(name = "accountId", required = true) Long accountId,
			@ApiParam(name = "code", value = "项目签到邀请码", required = true) @RequestParam(name = "code", required = true) String code,
			@ApiParam(name = "openId", value = "微信小程序openId", required = true) @RequestParam(name = "openId", required = true) String openId) {
		TrSignAccount signAccount = new TrSignAccount();
		signAccount.setAccountId(accountId);
		signAccount.setCode(code);
		signAccount.setOpenId(openId);
		signAccount.setDeleted(0);
		TrSignAccount one = signAccountService.selectOne(new EntityWrapper<TrSignAccount>(signAccount));
		if (one == null) {
			return "error";
		}
		one.setUpdateById(accountId);
		one.setUpdateTime(new Date());
		one.setDeleted(1);
		signAccountService.updateById(one);
		return "success";
	}

	@GetMapping("/insert")
	public String insert(
			@ApiParam(name = "accountId", value = "用户id ", required = true) @RequestParam(name = "accountId", required = true) Long accountId,
			@ApiParam(name = "trainingProjectId", value = "项目id ", required = true) @RequestParam(name = "trainingProjectId", required = true) Long trainingProjectId,
			@ApiParam(name = "signTimeId", value = "签到id ", required = true) @RequestParam(name = "signTimeId", required = true) Long signTimeId) {
		try {

			// 查询是否签到
			String result = signService.sign(accountId, String.valueOf(trainingProjectId),
					String.valueOf(signTimeId), 0);
			LOG.info(result);
			return result;
		} catch (Exception e) {
			LOG.error("", e);
			return "不存在此签到";
		}
	}
}
