package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.point.application.vo.PointParamVO;
import com.yizhi.sign.application.constant.SignRespCode;
import com.yizhi.application.domain.TrSign;
import com.yizhi.application.domain.TrSignRemind;
import com.yizhi.application.domain.TrSignTime;
import com.yizhi.point.application.feign.PointRedisFeignClients;
import com.yizhi.application.mapper.SignMapper;
import com.yizhi.application.mapper.SignRecordMapper;
import com.yizhi.application.mapper.SignRemindMapper;
import com.yizhi.application.mapper.SignTimeMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.service.SignService;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.application.util.SignEvenSendMessage;
import com.yizhi.util.application.date.DateUtil;
import com.yizhi.application.domain.TrSignRecord;
import com.yizhi.sign.application.vo.*;
import com.yizhi.sign.application.vo.domain.SignRecordVO;
import com.yizhi.training.application.feign.TrainingProjectClient;
import com.yizhi.training.application.vo.EvenType;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Pattern;


/**
 * <p>
 * 签到主体表，入口在培训项目 服务实现类
 * </p>
 *
 * @author fanchunhui123
 * @since 2018-03-07
 */
@Service
public class SignServiceImpl extends ServiceImpl<SignMapper, TrSign> implements SignService {

    @Autowired
    IdGenerator idGenerator;
    @Autowired
    SignMapper signMapper;
    @Autowired
    SignTimeMapper signTimeMapper;
    @Autowired
    SignRemindMapper signRemindMapper;

    @Autowired
	private SignRecordMapper signRecordMapper;
    @Autowired
	private TaskExecutor taskExecutor;
    @Autowired
	private SignEvenSendMessage signEvenSendMessage;
	@Autowired
	private TrainingProjectClient trainingProjectClient;
    @Autowired
    private PointRedisFeignClients pointRedisFeignClients;
    /*
    @Autowired
    private AmqpTemplate amqpTemplate;*/
    
    @Autowired
    private AccountClient accountClient;
	private Logger logger = LoggerFactory.getLogger(SignServiceImpl.class);

    @Transactional
    public boolean insertSignAndSignTimeAndSignRemind(SignVO signVO) {
        int signCount = 0;
        int signTimeCount = 0;
        int signRemindcount = 0;

        Long companyId = signVO.getCompanyId();
        Long siteId = signVO.getSiteId();
        Long accountId = signVO.getAccountId();
        String accountName = signVO.getAccountName();

       if(null != signVO){
    	   Date createTime = new Date();

    	   //添加签到数据
           TrSign trSign = new TrSign();
           BeanUtils.copyProperties(signVO,trSign);
           trSign.setCompanyId(companyId);
           trSign.setSiteId(siteId);
           trSign.setCreateById(accountId);
           trSign.setCreateByName(accountName);
           trSign.setCreateTime(createTime);
           trSign.setUpdateById(accountId);
           trSign.setUpdateByName(accountName);
           trSign.setUpdateTime(createTime);
           signCount = signMapper.insert(trSign);

           //添加签到时间段数据
           List<SignTimeVO> signTimeVOS = signVO.getTrSignTimeVOs();
           if(signTimeVOS != null && signTimeVOS.size() > 0 && signCount > 0){
        	   TrSignTime trSignTime = null;
        	   TrSignRemind trSignRemind = null;
        	   SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

               for (SignTimeVO signTimeVO : signTimeVOS) {
					try {
						trSignTime = new TrSignTime();
						trSignTime.setId(idGenerator.generate());
						trSignTime.setCreateById(accountId);
						trSignTime.setCreateByName(accountName);
						trSignTime.setCreateTime(createTime);
						trSignTime.setUpdateById(accountId);
						trSignTime.setUpdateByName(accountName);
						trSignTime.setUpdateTime(createTime);
						trSignTime.setTrainingProjectId(trSign.getTrainingProjectId());
						trSignTime.setSignId(trSign.getId());
						trSignTime.setDeleted(0);
						//拼接签到开始时间
						String sTime = signTimeVO.getStartTime();
						String sHour = signTimeVO.getStartHour();
						String sMinute = signTimeVO.getStartMinute();
						trSignTime.setStartTime(sdf.parse(sTime + " " + sHour + ":" + sMinute + ":00"));

						//拼接签到结束时间
						sTime = signTimeVO.getEndTime();
						sHour = signTimeVO.getEndHour();
						sMinute = signTimeVO.getEndMinute();
						trSignTime.setEndTime(sdf.parse(sTime + " " + sHour + ":" + sMinute + ":00"));
						trSignTime.setEnablePosition(signVO.getEnablePosition()!=null?signVO.getEnablePosition():0);
						trSignTime.setAddress(signTimeVO.getAddress());
						trSignTime.setRange(signTimeVO.getRange());
						trSignTime.setPoint(signTimeVO.getPoint());
						trSignTime.setLatitude(signTimeVO.getLatitude());
						trSignTime.setLongitude(signTimeVO.getLongitude());
						if (trSignTime.getEnablePosition()==1) {
							trSignTime.setCode(getStringRandom(5));
						}
						signTimeCount = signTimeMapper.insert(trSignTime);

						//添加消息提示
						if (signTimeCount > 0) {
							Date date = trSignTime.getStartTime();
							Integer remindTime = signVO.getRemindTime();
							Date beforeDate = null;
							if (null != remindTime && remindTime > 0) {
								if(null == trSignRemind) {
									trSignRemind = new TrSignRemind();
									trSignRemind.setCreateById(accountId);
									trSignRemind.setCreateByName(accountName);
									trSignRemind.setCreateTime(createTime);
									trSignRemind.setUpdateById(accountId);
									trSignRemind.setUpdateByName(accountName);
									trSignRemind.setUpdateTime(createTime);
								}
								trSignRemind.setSignTimeId(trSignTime.getId());
								trSignRemind.setId(idGenerator.generate());

								long time = remindTime * 60 * 1000;
								beforeDate = new Date(date.getTime() - time);
								String befacon = sdf.format(beforeDate);
								trSignRemind.setRemindTime(sdf.parse(befacon));

								signRemindcount = signRemindMapper.insert(trSignRemind);
							}
						}
					}catch (Exception e) {
                       e.printStackTrace();
                   }
               }
           }
       }
        if(signCount > 0 && signTimeCount > 0) {
            return true;
        }
        return false;
    }

    @Transactional
	@Override
    public boolean updateSignAndSignTimeAndSignRemind(SignVO signVO) {
        int signCount = 0;
        int signTimeCount = 0;
        int signRemindcount = 0;

        Long companyId = signVO.getCompanyId();
        Long siteId = signVO.getSiteId();
        Long accountId = signVO.getAccountId();
        String accountName = signVO.getAccountName();

        if(null != signVO){
        	Date updateTime = new Date();

            TrSign trSign = new TrSign();
            BeanUtils.copyProperties(signVO, trSign);
            trSign.setUpdateById(accountId);
            trSign.setUpdateByName(accountName);
            trSign.setUpdateTime(updateTime);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

            try {
                signCount = signMapper.updateById(trSign);
                if(signCount > 0) {
                    List<SignTimeVO> signTimeVOS = new ArrayList<SignTimeVO>();
                    TrSignTime trSignTime = null;

                    boolean isInsert = false;

                    signTimeVOS = signVO.getTrSignTimeVOs();
                    if (null != signTimeVOS) {
                        for (SignTimeVO signTimeVO : signTimeVOS) {
                            trSignTime = new TrSignTime();

                            //判断是新增还是修改
                            Long id = signTimeVO.getId();

                            if(id == null){
                            	isInsert = true;
                            	trSignTime.setId(idGenerator.generate());
                            	signTimeVO.setId(trSignTime.getId());
                            	trSignTime.setCreateById(accountId);
                            	trSignTime.setCreateByName(accountName);
                            	trSignTime.setCreateTime(updateTime);
                            	trSignTime.setSignId(trSign.getId());
                            	trSignTime.setDeleted(0);
								if (signVO.getEnablePosition() == 1) {
									trSignTime.setCode(getStringRandom(5));
								}
                            } else {
								TrSignTime tem = signTimeMapper.selectById(id);
								if (tem ==null){
									logger.error("查询签到的id无效！！！");
								}
								isInsert = false;
                            	trSignTime.setId(id);
								if (signVO.getEnablePosition() == 1) {
									if (tem.getEnablePosition() == 0) {
										trSignTime.setCode(getStringRandom(5));
									}
								}
                            }

                            trSignTime.setTrainingProjectId(trSign.getTrainingProjectId());
                            trSignTime.setUpdateById(accountId);
                            trSignTime.setUpdateByName(accountName);
                            trSignTime.setUpdateTime(updateTime);

                          //拼接签到开始时间
    						String sTime = signTimeVO.getStartTime();
    						String sHour = signTimeVO.getStartHour();
    						String sMinute = signTimeVO.getStartMinute();
    						trSignTime.setStartTime(sdf.parse(sTime + " " + sHour + ":" + sMinute + ":00"));

    						//拼接签到结束时间
    						sTime = signTimeVO.getEndTime();
    						sHour = signTimeVO.getEndHour();
    						sMinute = signTimeVO.getEndMinute();
    						trSignTime.setEndTime(sdf.parse(sTime + " " + sHour + ":" + sMinute + ":00"));
    						trSignTime.setEnablePosition(signVO.getEnablePosition()!=null?signVO.getEnablePosition():0);
    						trSignTime.setAddress(signTimeVO.getAddress());
    						trSignTime.setRange(signTimeVO.getRange());
    						trSignTime.setPoint(signTimeVO.getPoint());
    						trSignTime.setLatitude(signTimeVO.getLatitude());
    						trSignTime.setLongitude(signTimeVO.getLongitude());
    						if(isInsert){
    							 signTimeCount = signTimeMapper.insert(trSignTime);
    						} else {
    							 signTimeCount = signTimeMapper.updateById(trSignTime);
    						}
                            if (signTimeCount > 0) {
                                TrSignRemind trSignRemind = new TrSignRemind();
                                trSignRemind.setSignTimeId(trSignTime.getId());
                                Date date1 = trSignTime.getStartTime();

                                Integer remindTime = signVO.getRemindTime();
                                Date beforeDate = null;
                                if (null != remindTime && remindTime > 0) {
                                    long time1 = remindTime * 60 * 1000;
                                    beforeDate = new Date(date1.getTime() - time1);
                                    String befacon = sdf.format(beforeDate);
                                    trSignRemind.setRemindTime(sdf.parse(befacon));
                                }else {
									trSignRemind.setRemindTime(new Date());
								}

                                trSignRemind.setUpdateById(accountId);
                                trSignRemind.setUpdateByName(accountName);
                                trSignRemind.setUpdateTime(updateTime);
								if (isInsert) {
									trSignRemind.setId(idGenerator.generate());
									trSignRemind.setCreateById(accountId);
									trSignRemind.setCreateByName(accountName);
									trSignRemind.setCreateTime(updateTime);
									signRemindcount = signRemindMapper.insert(trSignRemind);
								} else {
									signRemindcount = signRemindMapper.updateBySignTimeId(trSignRemind);
								}
                            }
                        }

                        //修改删除签到状态
                        signTimeMapper.updateSignTimesStateBySignIdAndIds(signVO.getId(), signTimeVOS, accountId, accountName);
                    } else {
                    	 //修改删除签到状态
                        signTimeMapper.updateSignTimesStateBySignIdAndIds(signVO.getId(), null, accountId, accountName);
                    }
                } else {
                	//删除之前签到信息,添加签到信息
					signMapper.deleteSignByProjectId(signVO.getTrainingProjectId());
					TrSignTime trSignTime = new TrSignTime();
					trSignTime.setTrainingProjectId(signVO.getTrainingProjectId());
					signTimeMapper.delete(new EntityWrapper<>(trSignTime));
                	boolean isOk = insertSignAndSignTimeAndSignRemind(signVO);
                	if(isOk){
                		signCount = 1;
                		signTimeCount = 1;
                		signRemindcount = 1;
                	}
                }
            }catch (Exception e){
            	e.printStackTrace();
            }
        }
        if(signCount > 0 && signTimeCount > 0 && signRemindcount > 0){
            return true;
        }else{
            return false;
        }

    }
    @Override
    public Integer getSignTimeCount(SignVO signVO){
		Integer count = 0;
    	if(signVO != null ) {
			TrSignTime signTime = new TrSignTime();
			signTime.setSignId(signVO.getId());
			EntityWrapper wrapper = new EntityWrapper(signTime);
			List<TrSignTime> list = signTimeMapper.selectList(wrapper);
			if (!CollectionUtils.isEmpty(list)) {
				for (TrSignTime trSignTime : list) {
					if (trSignTime.getStartTime() != null && trSignTime.getEndTime() != null) {
						count++;
					}
				}
			}
		}
		return count;

	}


	/*************************************************************  PC 端  开始  *********************************************************/

	/**
	 * 获取签到总数量( PC 端   培训项目中培训统计的签到总数)
	 *
	 * @param trainingProjectId
	 * @return
	 */
	@Override
	public Integer getPcSignTimeCount(Long trainingProjectId){
		Integer count = 0;
		if(trainingProjectId != null ) {
			TrSignTime signTime = new TrSignTime();
			System.out.println("trainingProjectId为："+trainingProjectId);
			signTime.setTrainingProjectId(trainingProjectId);
			signTime.setDeleted(0);
			EntityWrapper wrapper = new EntityWrapper(signTime);
			List<TrSignTime> list = signTimeMapper.selectList(wrapper);
			if (!CollectionUtils.isEmpty(list)) {
				for (TrSignTime trSignTime : list) {
					if (trSignTime.getStartTime() != null && trSignTime.getEndTime() != null) {
						count++;
					}
				}
			}
		}
		return count;

	}

	/*************************************************************  PC 端  结束  *********************************************************/




	@Override
	public List<SignQrcodeVO> querySignTime(Long trainingProjectId) {

		//此处使用TrSign模拟 trainingProject  项目表目前不存在

		//查询签到表，获取签到时间id列表
		TrSignTime trSignTime = new TrSignTime();
		trSignTime.setTrainingProjectId(trainingProjectId);
		trSignTime.setDeleted(Integer.valueOf(0));
		List<TrSignTime> trSignTimeList = signTimeMapper.selectList(new EntityWrapper <TrSignTime> (trSignTime));
		List<SignQrcodeVO> signTimeIds = null;
		if(null != trSignTimeList && trSignTimeList.size()>0) {
			signTimeIds = new ArrayList<SignQrcodeVO> ();
			SignQrcodeVO vo = null;
			Iterator<TrSignTime> iterator = trSignTimeList.iterator();
			while(iterator.hasNext()) {
				TrSignTime tmp = (TrSignTime) iterator.next();
				vo = new SignQrcodeVO();
				vo.setId(tmp.getId());
				vo.setStartTime(tmp.getStartTime());
				vo.setEndTime(tmp.getEndTime());
				vo.setEnablePosition(tmp.getEnablePosition());
				vo.setAddress(tmp.getAddress());
				vo.setRange(tmp.getRange());
				vo.setCode(tmp.getCode());
				vo.setPoint(tmp.getPoint());
				signTimeIds.add(vo);
			}
		}

		return signTimeIds;

	}

	/**
	* @see SignService#queryQRCode(String)
	* @Function: SignServiceImpl.java
	* @Description:
	*
	* @param:
	* @return：
	* @throws：
	*
	* @author: xingliang
	* @date: 2018年3月8日 上午11:08:21
	*
	* 2018年3月8日     xingliang
	*/
	/*public String querySignTime(String trainingProjectId) {

		//此处使用TrSign模拟 trainingProject  项目表目前不存在
		TrSign trSign = new TrSign();
		trSign.setTrainingProjectId(Long.valueOf(trainingProjectId));
		trSign = signMapper.selectOne(trSign);

		//查询签到表，获取签到时间id列表
		TrSignTime trSignTime = new TrSignTime();
		trSignTime.setTrainingProjectId(Long.valueOf(trainingProjectId));
		trSignTime.setDeleted(Integer.valueOf(0));
		List<TrSignTime> trSignTimeList = signTimeMapper.selectList(new EntityWrapper <TrSignTime> (trSignTime));
		List<Long> signTimeIds = null;
		if(null != trSignTimeList && trSignTimeList.size()>0) {
			signTimeIds = new ArrayList<Long> ();

			Iterator<TrSignTime> iterator = trSignTimeList.iterator();
			while(iterator.hasNext()) {
				TrSignTime tmp = (TrSignTime) iterator.next();
				signTimeIds.add(tmp.getId());
			}
		}

		return JSON.toJSONString(signTimeIds);
	}*/


	/**
	* @see SignService#sign(String, Integer, String)
	* @Function: SignServiceImpl.java
	* @Description:
	*
	* @param:
	* @return：
	* @throws：
	*
	* @author: xingliang
	* @date: 2018年3月8日 下午2:37:09
	*
	* 2018年3月8日     xingliang
	*/
	public String sign(Long accountId, String trainingProjectId, String signTimeId,Integer type) {
		boolean flag = false;


		// 判断项目是否可以签到
		// 此处使用TrSign模拟 trainingProject  项目表目前不存在
		TrSign trSign = new TrSign();
		Long lTrainingProjectId = Long.valueOf(trainingProjectId);
		trSign.setTrainingProjectId(lTrainingProjectId);
		trSign.setState(0);
		trSign = signMapper.selectOne(trSign);

		String result = "";
		//判断项目是否允许签到
		if(null == trSign) {
			return result = SignRespCode.TRAINING_PROJECT_NULL_MSG;
		}

		// 判断项目是否可以补签 1是可以补签
		if(SignRespCode.SIGN_ENABLE_RETROACTIVE_TRUE == trSign.getEnableRetroactive()) {
			flag = true;
		}

		//判断签到时间是否在符合签到规则
		result = checkSign(accountId, trainingProjectId, signTimeId, flag, trSign,type);

		//签到成功发消息
		if (!result.contains("您已签到成功")&&(result.contains("补签成功")||result.contains("签到成功"))) {

			if (accountId != null) {
				TrainingProjectVo one = trainingProjectClient.getOne(lTrainingProjectId);
				try {
					if (trSign != null) {
						TrSign finalTrSign = trSign;
						if (finalTrSign != null) {
							taskExecutor.asynExecute(new AbstractTaskHandler() {
								@Override
								public void handle() {
									signEvenSendMessage.evenSendMessage(one, finalTrSign, accountId, EvenType.SIGN_SUCCESS);
								}
							});
						} else {
							logger.info("查询主体签到为空！！");
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
				TrSignTime signTime = signTimeMapper.selectById(signTimeId);
				Integer point = signTime.getPoint();
		        try {
		        	AccountVO accountVO = accountClient.findById(accountId);
		            PointParamVO pointParamVO = new PointParamVO();
		            pointParamVO.setEventName("pointProjectSign");    //事件名称
		            pointParamVO.setActivityType("项目签到成功");                //事件类型
		            pointParamVO.setActivitySource("项目签到");                            //学习活动来源
		            pointParamVO.setActivityName(one.getName());                //学习活动名称
		            pointParamVO.setReleaseCondition("指定积分发放");                    //发放条件
		            pointParamVO.setReleaseRules("项目签到成功发放");                    //发放规则
		            pointParamVO.setCreatePointTime(new Date());                            //积分创建时间
		            pointParamVO.setSourceId(Long.valueOf(signTimeId));                        //积分来源ID，（业务ID主键）
		            pointParamVO.setOperatingPoint(point);                //积分数量，  正数 加积分，  负数减积分
		            pointParamVO.setAccountId(accountId);        //学员ID
		            pointParamVO.setAccountName(accountVO.getName());    //学员名称
		            pointParamVO.setCompanyId(accountVO.getCompanyId());        //企业ID
		            pointParamVO.setSiteId(one.getSiteId());                //站点ID
		            pointParamVO.setOrgId(accountVO.getOrgId());                //组织ID
		            String sitePointId = null;
		            if (null != point && 0 != point) {
		                sitePointId = pointRedisFeignClients.addPointRedis(pointParamVO);    //调用积分接口添加积分
		            }
		            if (null != sitePointId) {
		               // amqpTemplate.convertAndSend("projectSign", sitePointId);    //发送积分获取消息
		            }
		        } catch (Exception e) {
		            logger.error("项目签到积分发放失败：", e);
		        }
			} else {
				logger.info("accountID为空！！！！");
			}
		}

		return result;
	}

	/**
	 * 判断签到规则
	* @Function: SignServiceImpl.java
	* @Description:
	*
	* @param:
	* @return：
	* @throws：
	*
	* @author: xingliang
	* @date: 2018年3月8日 下午2:49:08
	*
	* 2018年3月8日     xingliang
	 */
	@Transactional
	public String checkSign(Long accountId, String trainingProjectId, String signTimeId, boolean flag, TrSign trSign, Integer type) {
		// 判断签到时间是否在规定的签到时间以内
		TrSignTime trSignTime = new TrSignTime();
		trSignTime.setId(Long.valueOf(signTimeId));
		trSignTime.setTrainingProjectId(Long.valueOf(trainingProjectId));
		trSignTime.setDeleted(Integer.valueOf(0));
		trSignTime = signTimeMapper.selectOne(trSignTime);
		if (null==type) {
			type=0;
		}
		//签到时间能获取到
		if(null != trSignTime) {
			//判断是否签到
			TrSignRecord trSignRecord = new TrSignRecord();
			trSignRecord.setSignId(trSignTime.getId());
			trSignRecord.setAccountId(accountId);
			trSignRecord.setType(type);
			trSignRecord = signRecordMapper.selectOne(trSignRecord);
			if(trSignRecord != null){
				return "您已签到成功";
			}

			// 判断签到是否在规定的时间以内
			Date date = DateUtil.now();
			Date startTime = trSignTime.getStartTime();
			if(null == startTime){
				startTime = date;
			}
			Date endTime = trSignTime.getEndTime();
			if(null == endTime){
				endTime = date;
			}

			// 当前时间在规定的签到时间以内
			if(date.before(endTime) && date.after(startTime)) {
				// 签到表里插入记录
				boolean b = insertSignRecord(accountId, trSignTime, date, SignRespCode.SIGN_RECORD_STATUS_DEFAULT,type);
				if (b) {
					return "签到成功";
				}else {
					return "签到失败";
				}

			}

			// 当前时间晚于最终签到时间,并且项目允许补签
			if(date.after(endTime) && flag) {
				// 签到表里插入记录
				boolean b = insertSignRecord(accountId, trSignTime, date, SignRespCode.SIGN_RECORD_STATUS_RETROACTIVE,type);
				if (b) {
					trSignTime.setUpdateTime(new Date());
					signTimeMapper.updateById(trSignTime);
					return "补签成功";
				}else {
					return "补签失败";
				}
			}

			// 当前时间在最终签到时间之后,并且项目不允许补签
			if(date.after(endTime) && !flag) {
				return "签到失败,签到时间已过";
			}

			// 当前时间早于开始签到时间,不允许签到
			if(date.before(startTime)) {
				return "签到时间未开始";
			}
		} else {
			return "不存在此签到";
		}

		return "不存在此签到";
	}

	private boolean insertSignRecord(Long accountId, TrSignTime trSignTime, Date date, int signStatus, int type) {
		// 签到表里插入记录
		TrSignRecord trSignRecord = new TrSignRecord();
		trSignRecord.setId(idGenerator.generate());
		trSignRecord.setSignId(trSignTime.getId());
		trSignRecord.setSignTime(date);
		trSignRecord.setTrainingProjectId(trSignTime.getTrainingProjectId());
		trSignRecord.setSignStatus(signStatus);
		trSignRecord.setAccountId(accountId);
		trSignRecord.setType(type);
		trSignRecord.setCreateTime(date);
		trSignRecord.setAddress(trSignTime.getAddress());
		Integer i = signRecordMapper.insert(trSignRecord);
		if (i>0) {
			return true;
		}else {
			return false;
		}
	}


	/**
	 * 根据学员id信息,获取学员签到列表
	* @see SignService#signRecordList(Long)
	* @Function: SignServiceImpl.java
	* @Description:
	*
	* @param:
	* @return：
	* @throws：
	*
	* @author: xingliang
	* @date: 2018年3月9日 下午3:35:52
	*
	* 2018年3月9日     xingliang
	*/
	public List<SignRecordVO> signRecordList(Long accountId) {
		// TODO 查询签到列表

		return null;
	}

	/**
	 * 获取签到信息
	 * @param trainingProjectId 培训项目ID
	 * @return 签到对象
	 */
	public SignVO querySign(Long trainingProjectId) {
		//根据培训项目ID，查询项目
		TrSign trSign = new TrSign();
		trSign.setTrainingProjectId(trainingProjectId);
		trSign.setState(0);
		trSign = signMapper.selectOne(trSign);
		if(trSign == null){
			return null;
		}

		SignVO signVO = new SignVO();
		BeanUtils.copyProperties(trSign, signVO);

		//获取签到时间
		TrSignTime findSignTime = new TrSignTime();
		findSignTime.setTrainingProjectId(trainingProjectId);
		findSignTime.setSignId(signVO.getId());
		findSignTime.setDeleted(0);
		List<TrSignTime> trSignTime = signTimeMapper.selectList(new EntityWrapper<TrSignTime>(findSignTime));
		SignTimeVO signTimeVO = null;
		List<SignTimeVO> signTimeVOs = null;
		if(trSignTime != null){
			signTimeVOs = new ArrayList<SignTimeVO>();
			Date date = null;
			String dateArr[] = null;
			String timeArr[] = null;
			for(TrSignTime item : trSignTime){
				signTimeVO = new SignTimeVO();
				signTimeVO.setId(item.getId());

				//拆分签到开始时间
				date = item.getStartTime();
				String sDate = DateUtil.toMinute(date);
				dateArr = sDate.split(" ");
				signTimeVO.setStartTime(dateArr[0]);
				timeArr = dateArr[1].split(":");
				signTimeVO.setStartHour(timeArr[0]);
				signTimeVO.setStartMinute(timeArr[1]);

				//拆分签到结束时间
				date = item.getEndTime();
				sDate = DateUtil.toMinute(date);
				dateArr = sDate.split(" ");
				signTimeVO.setEndTime(dateArr[0]);
				timeArr = dateArr[1].split(":");
				signTimeVO.setEndHour(timeArr[0]);
				signTimeVO.setEndMinute(timeArr[1]);
				signTimeVO.setAddress(item.getAddress());
				signTimeVO.setRange(item.getRange());
				signTimeVO.setPoint(item.getPoint());
				signTimeVO.setLatitude(item.getLatitude());
				signTimeVO.setLongitude(item.getLongitude());
				signTimeVOs.add(signTimeVO);
			}
		}
		signVO.setTrSignTimeVOs(signTimeVOs);

		return signVO;
	}

	/**
	 * 获取我的签到
	 * @param accountId 学员ID
	 * @param month 月份为空查询所有
	 * @param pageNo 页码
	 * @param pageSize 页数
	 * @return
	 */
	public Page<TrSignRecordVo> selectSignRecordListByAccountIdAndMonth(Long accountId, String year, String month, int pageNo, int pageSize) {
		Page<TrSignRecordVo> page = new Page<TrSignRecordVo>(pageNo, pageSize);
		List<TrSignRecordVo> trSignRecordVos = signRecordMapper.selectSignRecordListByAccountIdAndMonth(page, accountId, year, month);
		page.setRecords(trSignRecordVos);

		return page;
	}

	@Override
	public boolean deleteSignByProjectId(SignIdVO signIdVO) {
		int count = signMapper.deleteSignByProjectId(signIdVO.getId());
		if (count > 0) {
			return true;
		} else {
			return false;
		}
	}

	public TrSign selectByTimeId(Long signTimeId){
		TrSign sign = signTimeMapper.selectByTimeId(signTimeId);
		return sign;
	}

	@Override
	public ApiSignTimeVO getSignTimeDeatil(Long accountId, String code, Long trainingProjectId) {
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Date date = new Date();
		ApiSignTimeVO vo = null;
		TrSignTime signTime = new TrSignTime();
		if (trainingProjectId!=null) {
			signTime.setTrainingProjectId(trainingProjectId);
		}
		signTime.setCode(code);
		signTime.setDeleted(0);
		TrSignTime one = signTimeMapper.selectOne(signTime);
		if (one!=null) {
			Date endTime = one.getEndTime();
			vo = new ApiSignTimeVO();
			Long tpId = one.getTrainingProjectId();
            TrainingProjectVo tProject = trainingProjectClient.getOne(tpId);
            if (tProject!=null) {
				vo.setTrainingProjectId(tProject.getId());
				vo.setName(tProject.getName());
			}
            vo.setId(one.getId());
            vo.setAddress(one.getAddress());
            vo.setEndTime(format.format(endTime));
            vo.setStartTime(format.format(one.getStartTime()));
            vo.setPoint(one.getPoint());
            vo.setRange(one.getRange());
            vo.setLatitude(one.getLatitude());
            vo.setLongitude(one.getLongitude());
            vo.setSiteId(tProject.getSiteId());
            TrSignRecord record = new TrSignRecord();
            record.setSignId(one.getId());
            record.setAccountId(accountId);
            TrSignRecord tRecord = signRecordMapper.selectOne(record);
            if (tRecord!=null) {
				vo.setState(1);
				vo.setSignDate(format.format(tRecord.getSignTime()));
			}else {
				vo.setState(0);
			}
		}
		return vo;
	}
	
    public static String getStringRandom(int length) {  
        String val = "";  
        Random random = new Random();  
        //参数length，表示生成几位随机数  
        for(int i = 0; i < length; i++) {  
           String charOrNum = random.nextInt(2) % 2 == 0 ? "char" : "num";  
            //输出字母还是数字  
          if("char".equalsIgnoreCase(charOrNum)){ 
              //输出是大写字母还是小写字母  
               int temp = random.nextInt(2) % 2 == 0 ? 65 : 97;  
                val += (char)(random.nextInt(26) + temp);  
           }else if("num".equalsIgnoreCase(charOrNum)) {  
                val += String.valueOf(random.nextInt(10));  
            }  
       }  
        String regex = "^(?=.*[a-z])(?=.*[A-Z])(?=.*\\d)[a-zA-Z\\d]{5,}$";
        boolean matches = Pattern.matches(regex, val);
        if (!matches||val.contains("l")||val.contains("l")||val.contains("I")||val.contains("O")||val.contains("0")) {
        	val = getStringRandom(5);
		}
        return val;  
    }
    
    public static void main(String[] args) {
		System.out.println(getStringRandom(5));
	}
}
