package com.yizhi.application.util;

import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.domain.TrSign;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.message.application.enums.RelationType;
import com.yizhi.training.application.vo.EvenType;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import com.yizhi.training.application.vo.manage.MessageRemindVo;
import com.yizhi.training.application.vo.manage.TaskVo;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

@Component
public class SignEvenSendMessage {


    @Autowired
    private CloudEventPublisher cloudEventPublisher;
    private Logger logger = LoggerFactory.getLogger(SignEvenSendMessage.class);


    /**
     * 触发事件,根据可见范围发消息
     *
     * @param trainingProject 业务参数对象
     * @param accountId
     */
    public void evenSendMessage(TrainingProjectVo trainingProject, TrSign finalTrSign, Long accountId, EvenType evenType) {
        if (finalTrSign.getCreateTime() == null) {
            logger.info("缺失必需参数！！");
            return;
        }
        TaskVo taskVo = new TaskVo();
        if (trainingProject != null) {
            taskVo.setTaskName(trainingProject.getName());
            taskVo.setTaskEndTime(finalTrSign.getCreateTime());
        } else {
            logger.info("查询出的培训为空！！！！");
            return;
        }
        MessageRemindVo vo = new MessageRemindVo();
        vo.setTaskVo(taskVo);
        vo.setVisibleRange(trainingProject.getVisibleRange());
        vo.setMessageId(evenType.getKey());
        vo.setMessageType(3);
        vo.setRelationId(finalTrSign.getId());
        vo.setRelationType(RelationType.QD.getKey());
        vo.setSendType(1);
        //定时任务默认五分钟执行一次   这里默认暂时不延迟时间
        vo.setSendTime(DateUtils.addMinutes(new Date(), 0));
        if (accountId != null) {
            vo.setAccountId(accountId);
        }
        RequestContext requestContext = new RequestContext();
        requestContext.setCompanyId(trainingProject.getCompanyId());
        requestContext.setSiteId(trainingProject.getSiteId());
        requestContext.setAccountId(trainingProject.getCreateById());
        requestContext.setAccountName(trainingProject.getCreateByName());
        vo.setRequestContext(requestContext);
        try {
            //临时取消触发功能
//            cloudEventPublisher.publish(Constans.MESSAGE_QUEUE, new EventWrapper<MessageRemindVo>(null, vo));
//            logger.info("发送消息成功===========");
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("发送消息失败=====================", e);
        }

    }
}
